<?php

	if ($GLOBALS["SOAP_USE_PHP5"] && function_exists("SoapServer")) {
		// Use PHP SOAP
		$server = new SoapServer();
		$server->addFunction("getInstitutes");
		$server->addFunction("getExternConfigurations");
		$server->addFunction("getPersonsAtInstitute");
		$server->addFunction("getLecturesAtInstitute");
		$server->addFunction("searchLectures");
		$server->addFunction("getLectureTypes");
		$server->addFunction("getSemesters");
		$server->addFunction("getSubjectAreasForInstitute");
		$server->addFunction("getTopicsOfStudyForInstitute");
		$server->addFunction("getGroupsForInstitute");
		$server->handle();

		// PHP5 heit: Neue Verzeichnisstruktur.
		require_once($STUDIP_BASE_PATH."/".$RELATIVE_PATH_EXTERN."/extern_config.inc.php");
		require_once($STUDIP_BASE_PATH."/config/config.inc.php");
		require_once($STUDIP_BASE_PATH."/lib/classes/SemesterData.class.php");
		require_once($STUDIP_BASE_PATH."/lib/classes/StudipSemTree.class.php");
	} else {
		// Use NuSOAP

		// New Stud.IP directory structure?
		if ($STUDIP_BASE_PATH) {
			require_once($STUDIP_BASE_PATH."/vendor/nusoap/nusoap.php");
			require_once($STUDIP_BASE_PATH."/".$RELATIVE_PATH_EXTERN."/extern_config.inc.php");
			require_once($STUDIP_BASE_PATH."/config/config.inc.php");
			require_once($STUDIP_BASE_PATH."/lib/classes/SemesterData.class.php");
			require_once($STUDIP_BASE_PATH."/lib/classes/StudipSemTree.class.php");
		} else {
			require_once($ABSOLUTE_PATH_STUDIP."/soap/lib/nusoap.php");
			require_once($ABSOLUTE_PATH_STUDIP."/extern/extern_config.inc.php");
			require_once($ABSOLUTE_PATH_STUDIP."/config.inc.php");
			require_once($ABSOLUTE_PATH_STUDIP."/lib/classes/SemesterData.class.php");
			require_once($ABSOLUTE_PATH_STUDIP."/lib/classes/StudipSemTree.class.php");
		}
		
		$server = new soap_server();
		$server->configureWSDL("studip_typo3_wsdl", "urn:studip_typo3");
		
		// Register all public functions
		$server->register("getInstitutes",
			array(),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getInstitutes",
			"rpc",
			"encoded",
			"Retrieves the hierarchy of institutes in Stud.IP."
		);
		$server->register("getExternConfigurations",
			array("rangeID" => "xsd:string"),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getExternConfigs",
			"rpc",
			"encoded",
			"Retrieves the external pages configurations for a given institute."
		);
		$server->register("getPersonsAtInstitute",
			array("rangeID" => "xsd:string"),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getPersonsAtInstitute",
			"rpc",
			"encoded",
			"Retrieves all persons with status \"dozent\" at an institute."
		);
		$server->register("getLecturesAtInstitute",
			array("rangeID" => "xsd:string"),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getLecturesAtInstitute",
			"rpc",
			"encoded",
			"Retrieves all lectures in current semester at an institute."
		);
		$server->register("searchLectures",
			array("searchtxt" => "xsd:string", "mode" => "xsd:string", "semester" => "xsd:string", "type" => "xsd:string", "institute" => "xsd:string"),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#searchLectures",
			"rpc",
			"encoded",
			"Searches for a given string in lectures or persons."
		);
		$server->register("getLectureTypes",
			array(),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getLectureTypes",
			"rpc",
			"encoded",
			"Retrieves all types of lectures defined in Stud.IP."
		);
		$server->register("getSemesters",
			array(),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getSemesters",
			"rpc",
			"encoded",
			"Retrieves all semesters defined in Stud.IP."
		);
		$server->register("getSubjectAreasForInstitute",
			array("rangeID" => "xsd:string"),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getSubjectAreasForInstitute",
			"rpc",
			"encoded",
			"Retrieves subject areas that are taught at an institute."
		);
		$server->register("getGroupsForInstitute",
			array("rangeID" => "xsd:string"),
			array("return" => "xsd:string"),
			"urn:studip_typo3",
			"urn:studip_typo3#getGroupsForInstitute",
			"rpc",
			"encoded",
			"Retrieves groups registered at an institute."
		);

		$HTTP_RAW_POST_DATA = isset($HTTP_RAW_POST_DATA) ? $HTTP_RAW_POST_DATA : "";
		$server->service($HTTP_RAW_POST_DATA);
	}
	
	/**
	 * Retrieves all institutes in the correct hierarchy.
	 * 
	 * @return An associative array containing the institute hierarchy.
	 */
	function getInstitutes() {
		$instituteArray = buildInstituteHierarchy();
		$result = serialize($instituteArray);
		return $result;
	}
	
	/**
	 * Retrieves all configurations for external pages registered at the 
	 * given institute. 
	 *
	 * @param string $rangeID an institute ID
	 * @return An array containing names and IDs of found configurations.
	 */
	function getExternConfigurations($rangeID) {
		global $EXTERN_MODULE_TYPES;
		$result = "";
		$configs = array();
		if (dbConnect()) {
			$query = "SELECT config_id, config_type, name FROM extern_config ".
				"WHERE range_id = '".$rangeID."' ORDER BY name";
			$data = mysql_query($query);
			if ($data) {
				while ($current = mysql_fetch_assoc($data)) {
					$configs[] = array("id" => $current["config_id"],
		 				"name" => replaceChars($current["name"]),
		 				"module" => replaceChars($EXTERN_MODULE_TYPES[
		 					$current["config_type"]]["module"]));
				}
			}
			$result = serialize($configs);
		}
		return $result;
	}
	
	/**
	 * Retrieves all persons which are registered at the given institute.
	 *
	 * @param string $rangeID an institute ID
	 * @return An associative array containing usernames and real names of 
	 * all found persons.
	 */
	function getPersonsAtInstitute($rangeID) {
		$result = "";
		$persons = array();
		if (dbConnect()) {
			$query = "SELECT DISTINCT username, TRIM(CONCAT(Nachname,', ',".
				"Vorname,IF(title_front!='',CONCAT(', ',title_front),''),".
				"IF(title_rear!='',CONCAT(', ',title_rear),''))) AS fullname ".
				"FROM statusgruppe_user LEFT JOIN auth_user_md5 aum ".
				"USING(user_id) LEFT JOIN user_info USING(user_id) LEFT JOIN ".
				"user_inst ui USING(user_id) WHERE statusgruppe_id IN ".
				"(SELECT statusgruppe_id FROM statusgruppen WHERE ".
				"range_id='".$rangeID."' ORDER BY position ASC) AND ".
				"Institut_id = '".$rangeID."' ORDER BY aum.Nachname, ".
				"aum.Vorname";
			$data = mysql_query($query);
			if ($data) {
				while ($current = mysql_fetch_assoc($data)) {
					$persons[] = array(
						"username" => replaceChars($current["username"]),
						"name" => replaceChars($current["fullname"]));
					}
				}
			$result = serialize($persons);
		}
		return $result;
	}
	
	/**
	 * Retrieves all courses which have the given institute as home 
	 * institute or participating institute.
	 *
	 * @param string $rangeID an institute ID
	 * @return An associative array containing all courses with their id, 
	 * names, semester and type.
	 */
	function getLecturesAtInstitute($rangeID) {
		global $SEM_TYPE, $SEM_TYPE_MISC_NAME;
		$result = "";
		$lectures = array();
		if (dbConnect()) {
			$query = "SELECT s.Seminar_id, s.Name, s.status, ".
				"sd.Name AS semester FROM seminar_inst si LEFT JOIN ".
				"seminare s ON (si.seminar_id=s.Seminar_id) LEFT JOIN ".
				"semester_data sd ON (s.start_time=sd.beginn) WHERE ".
				"si.Institut_id='".$rangeID."' AND s.Seminar_id IS NOT NULL ".
				"AND s.visible=1 ORDER BY s.Name";
			$data = mysql_query($query);
			if ($data) {
				while ($current = mysql_fetch_assoc($data)) {
					if ($SEM_TYPE[$current["status"]]["name"] == 
							$SEM_TYPE_MISC_NAME) {
						$type = $SEM_TYPE_MISC_NAME;
					} else {
						$type = $SEM_TYPE[$current["status"]]["name"];
					}
					$lectures[] = array(
						"id" => $current["Seminar_id"],
						"type" => replaceChars($type),
						"name" => replaceChars($current["Name"]),
						"semester" => replaceChars($current["semester"])
					);
				}
			}
			$result = serialize($lectures);
		}
		return $result;
	}
	
	/**
	 * Searches all courses by given filter criteria, such as semester or type.
	 *
	 * @param string $searchtxt the search string
	 * @param string $mode should the title, lecturer, comment or everything 
	 * be searched?
	 * @param int $semester search only courses beginning in this semester
	 * @param string $type search only courses of this type
	 * @param string $institute search only courses at this institute
	 * @return An associative array containing all found courses.
	 */
	function searchLectures($searchtxt, $mode, $semester, $type, $institute) {
		global $SEM_TYPE, $SEM_TYPE_MISC_NAME;
		$result = "";
		$lectures = array();
		if (dbConnect()) {
			if ($semester)
				$semesterClause = " AND s.start_time=".utf8_decode($semester);
			else
				$semesterClause = "";
			if ($type)
				$typeClause = " AND s.status=".utf8_decode($type);
			else
				$typeClause = "";
			if ($institute)
				$instituteClause = " AND s.Institut_id='".utf8_decode($institute)."'";
			else
				$instituteClause = "";
			switch ($mode) {
				// Search everything
				case "all":
					$query = "SELECT DISTINCT s.Seminar_id, s.Name, ".
						"s.status, sd.Name AS semester, s.Institut_id, " .
						"s.VeranstaltungsNummer AS lno FROM ".
						"seminare s JOIN semester_data sd ON ".
						"(s.start_time=sd.beginn) JOIN seminar_user su ON ".
						"(s.Seminar_id=su.Seminar_id AND su.status='dozent') ".
						"JOIN auth_user_md5 aum ON (aum.user_id=su.user_id) ".
						"WHERE (s.Name LIKE '%".utf8_decode($searchtxt).
						"%' OR aum.Nachname LIKE '%".utf8_decode($searchtxt).
						"%' OR s.Beschreibung LIKE '%".utf8_decode($searchtxt).
						"%' OR s.VeranstaltungsNummer LIKE '".utf8_decode($searchtxt).
						"') AND s.visible=1".$typeClause.$semesterClause.
						$instituteClause." ORDER BY s.start_time DESC, s.Name ASC";
					break;
				// Only search in course title
				case "title":
					$query = "SELECT DISTINCT s.Seminar_id, s.Name, ".
						"s.status, sd.Name AS semester, s.Institut_id, " .
						"s.VeranstaltungsNummer AS lno FROM ".
						"seminare s JOIN semester_data sd ON ".
						"(s.start_time=sd.beginn) JOIN seminar_user su ON ".
						"(s.Seminar_id=su.Seminar_id AND su.status='dozent') ".
						"JOIN auth_user_md5 aum ON (aum.user_id=su.user_id) ".
						"WHERE s.Name LIKE '%".utf8_decode($searchtxt)."%' AND s.visible=1".
						$typeClause.$semesterClause.
						$instituteClause." ORDER BY s.start_time DESC, s.Name ASC";
					break;
				// Only search in lecturer names
				case "lecturer":
					$query = "SELECT DISTINCT s.Seminar_id, s.Name, ".
						"s.status, sd.Name AS semester, s.Institut_id, " .
						"s.VeranstaltungsNummer AS lno FROM ".
						"seminare s JOIN semester_data sd ON ".
						"(s.start_time=sd.beginn) JOIN seminar_user su ON ".
						"(s.Seminar_id=su.Seminar_id AND su.status='dozent') ".
						"JOIN auth_user_md5 aum ON (aum.user_id=su.user_id) ".
						"WHERE aum.Nachname LIKE '%".utf8_decode($searchtxt)."%' AND ".
						"s.visible=1".$typeClause.$semesterClause." ORDER BY ".
						$instituteClause."s.start_time DESC, s.Name ASC";
					break;
				// Only search in course description
				case "description":
					$query = "SELECT DISTINCT s.Seminar_id, s.Name, s.status, ".
						"sd.Name AS semester, s.Institut_id, " .
						"s.VeranstaltungsNummer AS lno FROM ".
						"seminare s JOIN semester_data sd ON ".
						"(s.start_time=sd.beginn) JOIN seminar_user su ON ".
						"(s.Seminar_id=su.Seminar_id AND su.status='dozent') ".
						"JOIN auth_user_md5 aum ON (aum.user_id=su.user_id) ".
						"WHERE s.Beschreibung LIKE '%".utf8_decode($searchtxt)."%' AND ".
						"s.visible=1".$typeClause.$semesterClause." ORDER BY ".
						$instituteClause."s.start_time DESC, s.Name ASC";
					break;
				// Only search for course number
				case "number":
					$query = "SELECT DISTINCT s.Seminar_id, s.Name, s.status, ".
						"sd.Name AS semester, s.Institut_id, " .
						"s.VeranstaltungsNummer AS lno FROM ".
						"seminare s JOIN semester_data sd ON ".
						"(s.start_time=sd.beginn) JOIN seminar_user su ON ".
						"(s.Seminar_id=su.Seminar_id AND su.status='dozent') ".
						"JOIN auth_user_md5 aum ON (aum.user_id=su.user_id) ".
						"WHERE s.VeranstaltungsNummer LIKE '%".utf8_decode($searchtxt).
						"%' AND s.visible=1".$typeClause.$semesterClause.
						" ORDER BY ".$instituteClause."s.start_time DESC, ".
						"s.Name ASC";
					break;
				// Unspecified, do nothing
				default:
					$query = "SELECT * FROM seminare WHERE 1=0";
					break;
			}
			$data = mysql_query($query);
			// Build result array.
			if ($data) {
				while ($current = mysql_fetch_assoc($data)) {
					if ($SEM_TYPE[$current["status"]]["name"] == 
							$SEM_TYPE_MISC_NAME) {
						$type = $SEM_TYPE_MISC_NAME;
					} else {
						$type = $SEM_TYPE[$current["status"]]["name"];
					}
					$query2 = "SELECT DISTINCT aum.username, aum.Vorname, ".
						"aum.Nachname, ui.title_front FROM auth_user_md5 aum ".
						"JOIN seminar_user su ON (aum.user_id=su.user_id AND ".
						"su.status='dozent') JOIN user_info ui ON ".
						"(aum.user_id=ui.user_id) WHERE su.Seminar_id='".
						$current["Seminar_id"]."'";
					$data2 = mysql_query($query2);
					$lecturers = array();
					while ($lecturer = mysql_fetch_assoc($data2)) {
						$title = $lecturer["title_front"] ? ", ".
							$lecturer["title_front"] : "";
						$lecturers[] = array(
							"name" => replaceChars($lecturer["Nachname"]).", ".
									  replaceChars($lecturer["Vorname"]).
									  replaceChars($title),
							"username" => $lecturer["username"]
						);
					}
					$lectures[] = array(
						"id" => $current["Seminar_id"],
						"lno" => $current["lno"],
						"name" => replaceChars($current["Name"]),
						"type" => replaceChars($type),
						"semester" => replaceChars($current["semester"]),
						"lecturers" => $lecturers,
						"institute" => $current["Institut_id"]
					);
				}
			}
			$result = serialize($lectures);
		}
		return $result;
	}
	
	/**
	 * Retrieves all defined course types in Stud.IP.
	 *
	 * @return An associative array with the defined course types.
	 */
	function getLectureTypes() {
		global $SEM_TYPE, $SEM_CLASS;
		$result = array();
		for ($i=0 ; $i<=sizeof($SEM_CLASS) ; $i++) {
			$result[$SEM_CLASS[$i]["name"]] = array(); 
		}
		for ($i=1 ; $i<=sizeof($SEM_TYPE) ; $i++) {
			$result[$SEM_CLASS[$SEM_TYPE[$i]["class"]]["name"]][] = 
				array(
					"index" => $i,
					"name" => replaceChars($SEM_TYPE[$i]["name"])
				);
		}
		return serialize($result);
	}
	
	function getSemesters() {
		$semesters = SemesterData::getSemesterArray();
		return serialize(array_reverse($semesters));
	}
	
	function getSubjectAreasForInstitute($rangeID) {
		$result = "";
		if (dbConnect()) {
			$query = "SELECT DISTINCT st.sem_tree_id, st.name ". 
				"FROM sem_tree st JOIN seminar_sem_tree sst ON " .
				"(st.sem_tree_id=sst.sem_tree_id) JOIN seminar_inst si " .
				"ON (sst.seminar_id=si.seminar_id) WHERE si.institut_id='".
				$rangeID."'";
			$data = mysql_query($query);
			$semTree = new StudipSemTree(array("visible_only" => "1"));
			$subjectAreas = array();
			if ($data) {
				while ($current = mysql_fetch_assoc($data)) {
					$path = $semTree->getShortPath($current["sem_tree_id"]);
					$id = $current["sem_tree_id"];
					$name = $current["name"];
					$subjectAreas[] = array(
							"name" => replaceChars($name),
							"id" => $id,
							"path" => replaceChars($path)
						);
				}
			}
			$result = serialize($subjectAreas);
		}
		return $result;
	}
	
	function getTopicsOfStudyForInstitute($rangeID) {
		$result = "";
		if (dbConnect()) {
			$semTree = new StudipSemTree(array("visible_only" => "1"));
			$subjectAreas = array();
			if ($rangeID == 'root') {
				$semTreeID = 'root';
			} else {
				$semTreeID = getSemTreeID($rangeID);
			}
			$subjectAreas = buildSubjectAreaHierarchy($semTreeID, &$semTree);
			$result = serialize($subjectAreas);
		}
		return $result;
	}
	
	function getGroupsForInstitute($rangeID) {
		$result = "";
		if (dbConnect()) {
			$query = "SELECT sg.statusgruppe_id, sg.name FROM ".
				"statusgruppen sg JOIN statusgruppe_user su ON ".
				"(sg.statusgruppe_id = su.statusgruppe_id) ".
				"WHERE range_id = '".$rangeID."' GROUP BY sg.statusgruppe_id ".
				"HAVING COUNT(su.user_id)>0 ORDER BY sg.name";
			$data = mysql_query($query);
			$groups = array();
			if ($data) {
				while ($current = mysql_fetch_assoc($data)) {
					$groups[] = array(
							"name" => replaceChars($current["name"]),
							"id" => $current["statusgruppe_id"]
						);
				}
			}
			$result = serialize($groups);
		}
		return $result;
	}
	
	function dbConnect() {
		$host = $GLOBALS["DB_STUDIP_HOST"];
		$user = $GLOBALS["DB_STUDIP_USER"];
		$password = $GLOBALS["DB_STUDIP_PASSWORD"];
		$database = $GLOBALS["DB_STUDIP_DATABASE"];
		$connection = mysql_connect($host, $user, $password);
		if ($connection) {
			$selected_db = mysql_select_db($database);
			if ($selected_db)
				return true;
			else
				return false;
		} else {
			return false;
		}
	}
	
	function replaceChars($string) {
		$match = array("&", "<");
		$replace = array("&amp;", "&lt;");
		$string = str_replace($match, $replace, $string);
		$string = utf8_encode($string);
		return $string;
	}
	
	function buildInstituteHierarchy($parentID=false) {
		$result = array();
		if ($parentID) {
			if (dbConnect()) {
				$query = "SELECT DISTINCT rt.item_id, rt.parent_id, rt.name, ".
					"rt.studip_object_id, st.sem_tree_id FROM range_tree rt ".
					"LEFT JOIN sem_tree st ON ".
					"(rt.studip_object_id=st.studip_object_id) ".
					"WHERE rt.parent_id = '".$parentID."' ORDER BY rt.name";
				$data = mysql_query($query);
				if ($data) {
					while ($current = mysql_fetch_assoc($data)) {
						$subInstitutes = 
							buildInstituteHierarchy($current["item_id"]);
						if ($subInstitutes)
							$result[] = array("id" => $current["studip_object_id"],
								"name" => replaceChars($current["name"]),
								"subInstitutes" => $subInstitutes,
								"sem_tree_id" => $current["sem_tree_id"]);
						else
							$result[] = array("id" => $current["studip_object_id"],
								"name" => replaceChars($current["name"]),
								"sem_tree_id" => $current["sem_tree_id"]);
					}
				}
			}
		} else {
			$result[] = array("id" => "root",
				"name" => replaceChars($GLOBALS["UNI_NAME_CLEAN"]),
				"subInstitutes" => buildInstituteHierarchy("root"));
		}
		return $result;
	}
		
	function buildSubjectAreaHierarchy($start_id, &$sem_tree, $level=0) {
		$result = array();
		$children = $sem_tree->tree_childs[$start_id];
		foreach ($children as $child) {
			if ($sem_tree->tree_childs[$child]) {
				//$result = array_merge($result, buildSubjectAreaHierarchy($child, &$sem_tree));
				$result[] = array(
						'id' => $child,
						'name' => replaceChars($sem_tree->tree_data[$child]['name']),
						'path' => replaceChars($sem_tree->getShortPath($child), $level+1),
						'children' => buildSubjectAreaHierarchy($child, &$sem_tree, $level+1)
					);
			} else if (!$sem_tree->tree_data[$child]['studip_object_id']) {
				$result[] = array(
						'id' => $child,
						'name' => replaceChars($sem_tree->tree_data[$child]['name']),
						'path' => replaceChars($sem_tree->getShortPath($child), $level+1)
					);
			}
		}
		return $result;
	}
	
	function getSemTreeID($rangeID) {
		$result = '';
		if (dbConnect()) {
			$query = "SELECT sem_tree_id FROM sem_tree WHERE ".
				"studip_object_id='".$rangeID."' LIMIT 1";
			$data = mysql_query($query);
			if ($data) {
				$current = mysql_fetch_assoc($data);
				$result = $current['sem_tree_id'];
			}
		}
		return $result;
	}

?>
